#! /usr/bin/ruby

Dir.chdir(File.dirname($0))

PROJECT = "ChipmunkPro.xcodeproj"
IOS_SDK_VERSION = "5.0"
VERBOSE = true

require 'Tempfile'
BUILD_LOG = Tempfile.new("ChipmunkPro-")
BUILD_LOG_PATH = BUILD_LOG.path

def log(string)
	puts string
	open(BUILD_LOG_PATH, 'a'){|f| f.puts string}
end

def system(command)
	log "> #{command}"
	
	result = Kernel.system(VERBOSE ? "#{command} | tee -a #{BUILD_LOG_PATH}" : "#{command} >> #{BUILD_LOG_PATH}")
	
	if !result
		log "Command failed: #{command}"
		log "Build errors encountered. Aborting build script"
		log "Check the build log for more information: #{BUILD_LOG_PATH}"
		raise
	end
end

def build(target, configuration, simulator)
	sdk_os = (simulator ? "iphonesimulator" : "iphoneos")
	sdk = "#{sdk_os}#{IOS_SDK_VERSION}"
	
	command = "xcodebuild -project #{PROJECT} -sdk #{sdk} -configuration #{configuration} -target #{target}"
	system command
	
	return "build/#{configuration}-#{sdk_os}/lib#{target}.a"
end

def build_fat_lib(target, copy_list, trial=false)
	build_fat_lib(target, copy_list, false) if trial
	
	iphone_lib = build(target, (trial ? "Release-Trial" : "Release"), false)
	simulator_lib = build(target, "Debug", true)
	
	dirname = "#{target}-#{trial ? "iPhone-Trial" : "iPhone"}"
	
	system "rm -rf #{dirname}"
	system "mkdir #{dirname}"
	
	system "lipo #{iphone_lib} #{simulator_lib} -create -output #{dirname}/lib#{target}-iPhone.a"
	
	copy_list.each{|src| system "cp -r #{src} #{dirname}"}
end

build_fat_lib("ObjectiveChipmunk", ["Objective-Chipmunk/*.h", "Chipmunk/include/chipmunk"])

build_fat_lib("ChipmunkPro", [
	"Objective-Chipmunk/*.h",
	"AutoGeometry/*.h",
	"HastySpace/*.h",
	"Chipmunk/include/chipmunk",
], true)

BUILD_LOG.delete
